@@ -7,11 +7,14 @@
 
 
 
+#' @title
 #' Extract the Fixed Effects
 #' 
+#' @description 
 #' Function to extract the fixed effects from a `plm` object and
 #' associated summary method.
 #' 
+#' @details
 #' Function `fixef` calculates the fixed effects and returns an object
 #' of class `c("fixef", "numeric")`. By setting the `type` argument,
 #' the fixed effects may be returned in levels (`"level"`), as
@@ -19,6 +22,13 @@
 #' deviations from the overall mean (`"dmean"`). If the argument
 #' `vcov` was specified, the standard errors (stored as attribute "se"
 #' in the return value) are the respective robust standard errors.
+#' For two-way fixed-effect models, argument `effect` controls which
+#' of the fixed effects are to be extracted: `"individual"`, `"time"`, or 
+#' the sum of individual and time effects (`"twoways"`).
+#' NB: See **Examples** for how the sum of effects can be split in an individual
+#' and a time component.
+#' For one-way models, the effects of the model are extracted and the
+#' argument `effect` is disrespected.
 #' 
 #' The associated `summary` method returns an extended object of class
 #' `c("summary.fixef", "matrix")` with more information (see sections
@@ -28,45 +38,44 @@
 #' are, e.g., \insertCite{GREE:12;textual}{plm}, Ch. 11.4.4, p. 364,
 #' formulae (11-25); \insertCite{WOOL:10;textual}{plm}, Ch. 10.5.3,
 #' pp. 308-309, formula (10.58).
-#'
 #' @name fixef.plm
+#' @aliases fixef
 #' @param x,object an object of class `"plm"`, an object of class
 #'     `"fixef"` for the `print` and the `summary` method,
-#' @param effect one of `"individual"` or `"time"`, only relevant in
-#'     case of two--ways effects models,
+#' @param effect one of `"individual"`, `"time"`, or `"twoways"`, only relevant in
+#'     case of two--ways effects models (where it defaults to `"individual"`),
 #' @param vcov a variance--covariance matrix furnished by the user or
 #'     a function to calculate one (see **Examples**),
 #' @param type one of `"level"`, `"dfirst"`, or `"dmean"`,
 #' @param digits digits,
 #' @param width the maximum length of the lines in the print output,
 #' @param \dots further arguments.
-#' @return For function `fixef` an object of class `c("fixef",
-#'     "numeric")` is returned:\cr It is a numeric vector containing
+#' @return For function `fixef`, an object of class `c("fixef", "numeric")`
+#'     is returned: It is a numeric vector containing
 #'     the fixed effects with attribute `se` which contains the
 #'     standard errors. There are two further attributes: attribute
 #'     `type` contains the chosen type (the value of argument `type`
 #'     as a character); attribute `df.residual` holds the residual
 #'     degrees of freedom (integer) from the fixed effects model (plm
-#'     object) on which `fixef` was run.
-#' 
-#' For function `summary.fixef` an object of class `c("summary.fixef",
-#' "matrix")` is returned:\cr It is a matrix with four columns in this
-#' order: the estimated fixed effects, their standard errors and
-#' associated t--values and p--values.  The type of the fixed effects
-#' and the standard errors in the summary.fixef objects correspond to
-#' was requested in the `fixef` function by arguments `type` and
-#' `vcov`, respectively.
-#' @export
+#'     object) on which `fixef` was run. For the two-way unbalanced case, only
+#'     attribute `type` is added.
+#'     
+#' For function `summary.fixef`, an object of class
+#' `c("summary.fixef", "matrix")` is returned: It is a matrix with four
+#' columns in this order: the estimated fixed effects, their standard
+#' errors and associated t--values and p--values. 
+#' For the two-ways unbalanced case, the matrix contains only the estimates.
+#' The type of the fixed effects and the standard errors in the 
+#' summary.fixef object correspond to was requested in the `fixef`
+#' function by arguments `type` and `vcov`, respectively.
+#'     
 #' @author Yves Croissant
 #' @seealso [within_intercept()] for the overall intercept of fixed
 #'     effect models along its standard error, [plm()] for plm objects
 #'     and within models (= fixed effects models) in general. See
 #'     [ranef()] to extract the random effects from a random effects
 #'     model.
-#' @references
-#'
-#' \insertAllCited{}
-#'
+#' @references \insertAllCited{}
 #' @keywords regression
 #' @examples
 #' 
@@ -96,24 +105,54 @@
 #' 
 #' # calc. fitted values of oneway within model:
 #' fixefs <- fixef(gi)[index(gi, which = "id")]
-#' fitted_by_hand <- fixefs + gi$coefficients["value"] * gi$model$value +
+#' fitted_by_hand <- fixefs + gi$coefficients["value"]   * gi$model$value +
 #'                            gi$coefficients["capital"] * gi$model$capital
+#'
+#' # calc. fittes values of twoway unbalanced within model via effects:
+#' gtw_u <- plm(inv ~ value + capital, data = Grunfeld[-200, ], effect = "twoways")
+#' yhat <- as.numeric(gtw_u$model[ , 1] - gtw_u$residuals) # reference
+#' pred_beta <- as.numeric(tcrossprod(coef(gtw_u), as.matrix(gtw_u$model[ , -1])))
+#' pred_effs <- as.numeric(fixef(gtw_u, "twoways")) # sum of ind and time effects
+#' all.equal(pred_effs + pred_beta, yhat) # TRUE
 #' 
+#' # Splits of summed up individual and time effects:
+#' # use one "level" and one "dfirst"
+#' ii <- index(gtw_u)[[1L]]; it <- index(gtw_u)[[2L]]
+#' eff_id_dfirst <- c(0, as.numeric(fixef(gtw_u, "individual", "dfirst")))[ii]
+#' eff_ti_dfirst <- c(0, as.numeric(fixef(gtw_u, "time",       "dfirst")))[it]
+#' eff_id_level <- as.numeric(fixef(gtw_u, "individual"))[ii]
+#' eff_ti_level <- as.numeric(fixef(gtw_u, "time"))[it]
+#' 
+#' all.equal(pred_effs, eff_id_level  + eff_ti_dfirst) # TRUE
+#' all.equal(pred_effs, eff_id_dfirst + eff_ti_level)  # TRUE
+#'
+#' @importFrom nlme fixef
+#' @export fixef
+NULL
+
+#' @rdname fixef.plm
+#' @importFrom stats weighted.mean
+#' @export
 fixef.plm <- function(object, effect = NULL,
                       type = c("level", "dfirst", "dmean"),
                       vcov = NULL, ...){
     
     model.effect <- describe(object, "effect")
-    if (is.null(effect)){
-        effect <- ifelse(model.effect == "time", "time", "individual")
+    if(is.null(effect)){
+        # default for twoway model to individual effect
+        effect <- switch(model.effect,
+                           "individual" = "individual",
+                           "time"       = "time",
+                           "twoways"    = "individual")
     }
     else{
-        if (! effect %in% c("individual", "time")) stop("wrong effect argument")
-        if (model.effect != "twoways" && model.effect != effect) stop("wrong effect argument")
+        if(model.effect != "twoways" && model.effect != effect) stop("wrong effect argument")
+        if(!effect %in% c("individual", "time", "twoways")) stop("wrong effect argument")
     }
+    
     type <- match.arg(type)
-    if (!is.null(object$call)){
-        if (describe(object, "model") != "within")
+    if(!is.null(object$call)){
+        if(describe(object, "model") != "within")
       stop("fixef is relevant only for within models")
     }
     formula <- formula(object)
@@ -123,48 +162,113 @@ fixef.plm <- function(object, effect = NULL,
     # within model doesn't. So select the relevant elements using nw
     # (names of the within variables)
     nw <- names(coef(object))
-  
-    # For procedure to get the individual/time effects by muliplying the within
-    # estimates with the between-ed data, see e.g.
+    
+    # For procedure to get the individual/time effects by multiplying the within
+    # estimates with the between-ed data, see, e.g.:
     #  Wooldridge (2010), Econometric Analysis of Cross Section and Panel Data, 2nd ed., 
     #                     Ch. 10.5.3, pp. 308-309, formula (10.58)
     #  Greene (2012), Econometric Analysis,
     #                 Ch. 11.4.4, p. 364, formulae (11-25)
     #
-    # NB: These formulae do not give the correct results in the two-ways unbalanced case,
-    #     all other cases (twoways/balanced; oneway(ind/time)/balanced/unbalanced) seem to
-    #     work with these formulae.
-    Xb <- model.matrix(data, rhs = 1, model = "between", effect = effect)
-    yb <- pmodel.response(data, model = "between", effect = effect)
-    fixef <- yb - as.vector(crossprod(t(Xb[, nw, drop = FALSE]), coef(object)))
-  
-    # use robust vcov if supplied
-    if (! is.null(vcov)) {
+    # NB: These textbook formulae do not give the correct results in the two-ways unbalanced case,
+    #     all other cases (twoways/balanced; oneway(ind/time)/balanced/unbalanced) are correct
+    #     for these formulae.
+    if(model.effect != "twoways") {
+      Xb <- model.matrix(data, rhs = 1, model = "between", effect = effect)
+      yb <- pmodel.response(data, model = "between", effect = effect)
+      fixef <- yb - as.vector(crossprod(t(Xb[ , nw, drop = FALSE]), coef(object)))
+      
+      # use robust vcov if supplied
+      if (! is.null(vcov)) {
         if (is.matrix(vcov))   vcov <- vcov[nw, nw]
         if (is.function(vcov)) vcov <- vcov(object)[nw, nw]
-    } else {
+      } else {
         vcov <- vcov(object)[nw, nw]
-    }
-
-    nother <- switch(effect,
-                     "individual" = pdim$Tint$Ti,
-                     "time"       = pdim$Tint$nt)
-  
-    s2 <- deviance(object) / df.residual(object)
-    if (type != "dfirst") {
-        sefixef <- sqrt(s2 / nother + apply(Xb[, nw, drop = FALSE],1,function(x) t(x) %*% vcov %*% x))
-    } else {
+      }
+      
+      nother <- switch(effect,
+                       "individual" = pdim$Tint$Ti,
+                       "time"       = pdim$Tint$nt)
+      
+      s2 <- deviance(object) / df.residual(object)
+      if (type != "dfirst") {
+        sefixef <- sqrt(s2 / nother + apply(Xb[, nw, drop = FALSE], 1,
+                                            function(x) t(x) %*% vcov %*% x))
+      } else {
         Xb <- t(t(Xb[-1, ]) - Xb[1, ])
         sefixef <- sqrt(s2 * (1 / nother[-1] + 1 / nother[1])+
-                        apply(Xb[, nw, drop = FALSE],1,function(x) t(x) %*% vcov %*% x))
+                          apply(Xb[, nw, drop = FALSE], 1,
+                                function(x) t(x) %*% vcov %*% x))
+      }
+      fixef <- switch(type,
+                      "level"  = fixef,
+                      "dfirst" = fixef[2:length(fixef)] - fixef[1],
+                      "dmean"  = (fixef - weighted.mean(fixef, w = nother)))
+      
+      res <- structure(fixef, se = sefixef, class = c("fixef", "numeric"),
+                type = type, df.residual = df.residual(object))  
+      return(res)
     }
     
-    fixef <- switch(type,
-                    "level"  = fixef,
-                    "dfirst" = fixef[2:length(fixef)] - fixef[1],
-                    "dmean"  = fixef - mean(fixef)
-                    )
-    structure(fixef, se = sefixef, class = c("fixef", "numeric"), type = type, df.residual = df.residual(object))
+    ## treat separately here:
+    ##  * two-way balanced/unbalanced model for all effects
+    if(model.effect == "twoways") {
+      beta.data <- as.numeric(tcrossprod(coef(object), as.matrix(object$model[ , -1, drop = FALSE])))
+      yhat <- object$model[ , 1] - object$residuals
+      tw.fixef.lvl <- yhat - beta.data # sum of both effects in levels
+      
+      idx <- switch(effect,
+                    "individual" = 1L,
+                    "time"       = 2L,
+                    "twoways"    = NA_integer_) # needed for weighted.mean below -> leads to no weights
+
+      if(effect %in% c("individual", "time")) {
+        other.eff <- switch(effect,
+                            "individual" = "time",
+                            "time"       = "individual")
+        
+        other.idx <- switch(effect,
+                          "individual" = 2L,
+                          "time"       = 1L)
+
+        Xb <- model.matrix(data, rhs = 1, model = "between", effect = other.eff)
+        yb <- pmodel.response(data, model = "between", effect = other.eff)
+        other.fixef.lvl <- yb - as.vector(crossprod(t(Xb[ , nw, drop = FALSE]), coef(object)))
+        
+        ## other dmean
+        #other.fixef.dmean <- other.fixef.lvl - mean(other.fixef.lvl)
+        #other.fixef.dmean <- other.fixef.dmean[index(object)[[other.idx]]]
+        #tw.fixef.lvl <- tw.fixef.lvl - other.fixef.dmean
+        
+        ## other level
+        # tw.fixef.lvl <- tw.fixef.lvl - other.fixef.lvl[index(object)[[other.idx]]]
+        
+        ## other dfirst
+        other.fixef.dfirst <- other.fixef.lvl - other.fixef.lvl[1L]
+        tw.fixef.lvl <- tw.fixef.lvl - other.fixef.dfirst[index(object)[[other.idx]]]
+        
+        tw.fixef.lvl <- tw.fixef.lvl[!duplicated(index(object)[[idx]])]
+        names(tw.fixef.lvl) <- pdim[["panel.names"]][[idx]]
+      } else {
+        # everything already computed, just set names
+        names(tw.fixef.lvl) <- paste0(pdim[["panel.names"]][[1L]][index(object)[[1L]]], "-",
+                                      pdim[["panel.names"]][[2L]][index(object)[[2L]]])
+      }
+      
+      tw.fixef <- switch(type,
+                      "level"  = tw.fixef.lvl,
+                      "dfirst" = tw.fixef.lvl[2:length(tw.fixef.lvl)] - tw.fixef.lvl[1],
+                      "dmean"  = {
+                          if(pdim$balanced || effect == "twoways") {
+                            tw.fixef.lvl - mean(tw.fixef.lvl) 
+                          } else {
+                            tw.fixef.lvl - weighted.mean(tw.fixef.lvl, w = pdim$Tint[[idx]])
+                          }})
+      
+      tw.fixef <- structure(tw.fixef, se = NULL, class = c("fixef", "numeric"),
+                            type = type, df.residual = NULL)
+      return(tw.fixef)
+    }
 }
 
 
@@ -173,7 +277,7 @@ fixef.plm <- function(object, effect = NULL,
 print.fixef <- function(x, digits = max(3, getOption("digits") - 2),
                         width = getOption("width"), ...){
   
-    # prevent attributs from being printed
+    # prevent attributes from being printed
     attr(x, "se") <- attr(x, "type") <- attr(x, "class") <- attr(x, "df.residual") <- attr(x, "index") <- NULL
     print.default(x, digits, width, ...)
 }
@@ -181,7 +285,9 @@ print.fixef <- function(x, digits = max(3, getOption("digits") - 2),
 
 #' @rdname fixef.plm
 #' @export
-summary.fixef <- function(object, ...){
+summary.fixef <- function(object, ...) {
+  # for 2-way unbalanced, there are currently no further attributes -> skip construction
+  res <- if(!is.null(attr(object, "se"))) {
     se <- attr(object, "se")
     df.res <- attr(object, "df.residual")
     tvalue <- (object) / se
@@ -195,6 +301,10 @@ summary.fixef <- function(object, ...){
     attr(res, "type") <- attr(object, "type")
     attr(res, "df.residual") <- df.res
     res
+  } else {
+    matrix(object, dimnames = list(names(object), "Estimate"))
+  }
+  res
 }
 
 #' @rdname fixef.plm
@@ -235,7 +345,10 @@ fixef.pggls <- fixef.plm
 #' @param \dots further arguments (currently not used).
 #' @return A named numeric with the random effects per dimension
 #'     (individual or time).
-#' @export
+#' @name ranef.plm
+#' @aliases ranef
+#' @importFrom nlme ranef
+#' @export ranef
 #' @author Kevin Tappe
 #' @seealso [fixef()] to extract the fixed effects from a fixed
 #'     effects model (within model).
@@ -246,28 +359,27 @@ fixef.pggls <- fixef.plm
 #' m1 <- plm(inv ~ value + capital, data = Grunfeld, model = "random")
 #' ranef(m1) # individual random effects
 #' 
-#' # compare to random effects by ML estimation via lmer from package
-#' # lme4
-#' \dontrun{
-#'   library(lme4)
-#'   m2 <- lmer(inv ~ value + capital + (1 | firm), data = Grunfeld) 
-#'   cbind("plm" = ranef(m1), "lmer" = unname(ranef(m2)$firm))
-#' }
+#' # compare to random effects by ML estimation via lme from package nlme
+#' library(nlme)
+#' m2 <- lme(inv ~ value + capital, random = ~1|firm, data = Grunfeld)
+#' cbind("plm" = ranef(m1), "lme" = unname(ranef(m2)))
 #' 
 #' # two-ways RE model, calculate individual and time random effects
 #' data("Cigar", package = "plm")
 #' tw <- plm(sales ~ pop + price, data = Cigar, model = "random", effect = "twoways")
 #' ranef(tw)                   # individual random effects
 #' ranef(tw, effect = "time")  # time random effects
-#' 
 #'
-#'
-# TODO:
-#      Check if the same procedure can be applied to
-#       * unbalanced two-way case (for now: implemented the same way, but not entirely sure)
-#       * random IV models
-#       * nested random effect models
+NULL
+
+#' @rdname ranef.plm
+#' @export
 ranef.plm <- function(object, effect = NULL, ...) {
+  # TODO:
+  #      Check if the same procedure can be applied to
+  #       * unbalanced two-way case (for now: implemented the same way, but not entirely sure)
+  #       * random IV models
+  #       * nested random effect models
   model <- describe(object, "model")
   obj.effect <- describe(object, "effect")
   balanced <- is.pbalanced(object)
@@ -334,20 +446,17 @@ ranef.plm <- function(object, effect = NULL, ...) {
 
 
 
-# generic
-
-
 #' Overall Intercept for Within Models Along its Standard Error
 #' 
 #' This function gives an overall intercept for within models and its
-#' accompanying standard error
+#' accompanying standard error or an within model with the overall intercept
 #' 
 #' The (somewhat artificial) intercept for within models (fixed
 #' effects models) was made popular by Stata of StataCorp
 #' \insertCite{@see @GOUL:13}{plm}, EViews of IHS, and gretl
 #' \insertCite{@gretl p. 160-161, example 18.1}{plm}, see for
 #' treatment in the literature,
-#' e.g. \insertCite{GREE:12;textual}{plm}, Ch. 11.4.4, p. 364. It can
+#' e.g., \insertCite{GREE:12;textual}{plm}, Ch. 11.4.4, p. 364. It can
 #' be considered an overall intercept in the within model framework
 #' and is the weighted mean of fixed effects (see **Examples** for the
 #' relationship).
@@ -361,21 +470,30 @@ ranef.plm <- function(object, effect = NULL, ...) {
 #' Users can set argument `vcov` to a function to calculate a
 #' specific (robust) variance--covariance matrix and get the
 #' respective (robust) standard error for the overall intercept,
-#' e.g. the function [vcovHC()], see examples for
+#' e.g., the function [vcovHC()], see examples for
 #' usage. Note: The argument `vcov` must be a function, not a
 #' matrix, because the model to calculate the overall intercept for
 #' the within model is different from the within model itself.
 #' 
+#' If argument `return.model = TRUE` is set, the full model object is returned,
+#' while in the default case only the intercept is returned.
+#' 
 #' @aliases within_intercept
 #' @param object object of class `plm` which must be a within
 #'     model (fixed effects model),
 #' @param vcov if not `NULL` (default), a function to calculate a
 #'     user defined variance--covariance matrix (function for robust
-#'     vcov),
+#'     vcov), only used if `return.model = FALSE`,
+#' @param return.model a logical to indicate whether only the overall intercept
+#'     (`FALSE` is default) or a full model object (`TRUE`) is to be returned, 
 #' @param \dots further arguments (currently none).
-#' @return A named `numeric` of length one: The overall intercept
-#'     for the estimated within model along attribute "se" which
-#'     contains the standard error for the intercept.
+#' @return Depending on argument `return.model`:  If `FALSE` (default), a named
+#' `numeric` of length one: The overall intercept for the estimated within model
+#'  along attribute "se" which contains the standard error for the intercept.
+#'  If `return.model = TRUE`, the full model object, a within model with the
+#'  overall intercept (NB: the model identifies itself as a pooling model, e.g.,
+#'  in summary()).
+#'  
 #' @export
 #' @author Kevin Tappe
 #' @seealso [fixef()] to extract the fixed effects of a
@@ -386,10 +504,6 @@ ranef.plm <- function(object, effect = NULL, ...) {
 #'
 #' @keywords attribute
 #' @examples
-#' 
-# TODO: two-way unbalanced case, once fixef() produces results that
-# are compatible estimate within model (unbalanced data)
-#
 #' data("Hedonic", package = "plm")
 #' mod_fe <- plm(mv ~ age + crim, data = Hedonic, index = "townid")
 #' overallint <- within_intercept(mod_fe)
@@ -397,20 +511,34 @@ ranef.plm <- function(object, effect = NULL, ...) {
 #' 
 #' # overall intercept is the weighted mean of fixed effects in the
 #' # one-way case
-#' weighted.mean(fixef(mod_fe), as.numeric(table(index(mod_fe)[[1]])))
+#' weighted.mean(fixef(mod_fe), pdim(mod_fe)$Tint$Ti)
 #' 
-#' # relationship of type="dmean", "level" and within_intercept in the
-#' # one-way case
+#' ### relationship of type="dmean", "level" and within_intercept
+#' ## one-way balanced case
 #' data("Grunfeld", package = "plm")
 #' gi <- plm(inv ~ value + capital, data = Grunfeld, model = "within")
 #' fx_level <- fixef(gi, type = "level")
 #' fx_dmean <- fixef(gi, type = "dmean")
 #' overallint <- within_intercept(gi)
 #' all.equal(overallint + fx_dmean, fx_level, check.attributes = FALSE) # TRUE
+#' ## two-ways unbalanced case
+#' gtw_u <- plm(inv ~ value + capital, data = Grunfeld[-200, ], effect = "twoways")
+#' int_tw_u <- within_intercept(gtw_u)
+#' fx_dmean_tw_i_u <- fixef(gtw_u, type = "dmean", effect = "individual")[index(gtw_u)[[1L]]]
+#' fx_dmean_tw_t_u <- fixef(gtw_u, type = "dmean", effect = "time")[index(gtw_u)[[2L]]]
+#' fx_level_tw_u <- as.numeric(fixef(gtw_u, "twoways", "level"))
+#' fx_level_tw_u2 <- int_tw_u + fx_dmean_tw_i_u + fx_dmean_tw_t_u
+#' all.equal(fx_level_tw_u, fx_level_tw_u2, check.attributes = FALSE) # TRUE
 #' 
-#' # overall intercept with robust standard error
+#' ## overall intercept with robust standard error
 #' within_intercept(gi, vcov = function(x) vcovHC(x, method="arellano", type="HC0"))
-#' 
+#'
+#' ## have a model returned
+#' mod_fe_int <- within_intercept(gi, return.model = TRUE)
+#' summary(mod_fe_int)
+#' # replicates Stata's robust standard errors
+#' summary(mod_fe_int, vcvov = function(x) vcovHC(x, type = "sss")) 
+# 
 within_intercept <- function(object, ...) {
   UseMethod("within_intercept")
 }
@@ -426,8 +554,8 @@ within_intercept <- function(object, ...) {
 
 #' @rdname within_intercept
 #' @export
-within_intercept.plm <- function(object, vcov = NULL, ...) {
-  
+within_intercept.plm <- function(object, vcov = NULL, return.model = FALSE, ...) {
+  # TODO: check 2-way FE case -> is ok for balanced case
   if (!inherits(object, "plm")) stop("input 'object' needs to be a \"within\" model estimated by plm()")
   model  <- describe(object, what = "model")
   effect <- describe(object, what = "effect")
@@ -435,11 +563,11 @@ within_intercept.plm <- function(object, vcov = NULL, ...) {
   
   # vcov must be a function, because the auxiliary model estimated to get the
   # overall intercept next to its standard errors is different from
-  # the FE model for which the intercept is estimated, e.g. dimensions
+  # the FE model for which the intercept is estimated, e.g., dimensions
   # of vcov differ for FE and for auxiliary model.
-  if (!is.null(vcov)) {
-    if (is.matrix(vcov)) stop("for within_intercept, 'vcov' may not be of class 'matrix', it must be supplied as a function, e.g. vcov = function(x) vcovHC(x)")
-    if (!is.function(vcov)) stop("for within_intercept, argument 'vcov' must be a function, e.g. vcov = function(x) vcovHC(x)")
+  if(!is.null(vcov)) {
+    if(is.matrix(vcov)) stop("for within_intercept, 'vcov' may not be of class 'matrix', it must be supplied as a function, e.g., vcov = function(x) vcovHC(x)")
+    if(!is.function(vcov)) stop("for within_intercept, argument 'vcov' must be a function, e.g., vcov = function(x) vcovHC(x)")
   }
   
   index <- attr(object$model, which = "index")
@@ -464,33 +592,34 @@ within_intercept.plm <- function(object, vcov = NULL, ...) {
 
   # estimation by plm() - to apply robust vcov function if supplied
   data <- pdata.frame(data.frame(cbind(index, transY, transM)), drop.index = TRUE)
-  form <- as.formula(paste0(names(data)[1], "~", paste(names(data)[-1], collapse = "+")))
+  form <- as.formula(paste0(names(data)[1], "~", paste(names(data)[-1L], collapse = "+")))
   auxreg <- plm(form, data = data, model = "pooling")
   
-  ## Two cases:
-  ##  (1) in case of "normal" vcov, we need to adjust the vcov by the corrected degrees of freedom
-  ##  (2) in case of robust vcov, which is supplied by a function, no adjustment to the robust vcov is necessary
-  if (!is.function(vcov)) {
-    # (1) degrees of freedom correction due to FE transformation for "normal" vcov [copied over from plm.fit]
-    pdim <- pdim(index)
-    card.fixef <- switch(effect,
-                            "individual" = pdim$nT$n,
-                            "time"       = pdim$nT$T,
-                            "twoways"    = pdim$nT$n + pdim$nT$T - 1)
-    df <- df.residual(auxreg) - card.fixef  + 1 # just for within_intercept: here we need '+1' to correct for the intercept
-    
-    vcov_new <- vcov(auxreg)
-    vcov_new <- vcov_new * df.residual(auxreg) / df
+  # degrees of freedom correction due to FE transformation for "normal" vcov [copied over from plm.fit]
+  pdim <- pdim(index)
+  card.fixef <- switch(effect,
+                       "individual" = pdim$nT$n,
+                       "time"       = pdim$nT$T,
+                       "twoways"    = pdim$nT$n + pdim$nT$T - 1L)
+  df <- df.residual(auxreg) - card.fixef  + 1 # just for within_intercept: here we need '+1' to correct for the intercept
+  
+  vcov_mat <- vcov(auxreg)
+  vcov_mat <- vcov_mat * df.residual(auxreg) / df
+  auxreg$vcov <- vcov_mat # plug in new vcov (adjusted "normal" vcov) in auxiliary model
+
+  res <- if(!return.model) {
+    #### return only intercept with SE as attribute
+    ##  in case of robust vcov, which is supplied by a function
+    ##  no adjustment to the robust vcov is necessary
+    if(is.function(vcov)) vcov_mat <- vcov(auxreg) # robust vcov as supplied by a function
+    intercept <- auxreg[["coefficients"]]["(Intercept)"]
+    attr(intercept, which = "se") <- sqrt(vcov_mat[1L, 1L])
+    names(intercept) <- "(overall_intercept)"
+    intercept
   } else {
-    # (2) robust vcov estimated by function supplied in vcov
-      vcov_new <- vcov(auxreg)
+    ### return model
+    if(!is.null(vcov)) warning("argument 'vcov' is non-NULL and is ignored as 'return.model = TRUE' is set")
+    auxreg
   }
-  
-  auxreg$vcov <- vcov_new # plug in new vcov (adjusted "normal" vcov or robust vcov) in auxiliary model
-  
-  coef_se <- lmtest::coeftest(auxreg)
-  intercept <- coef_se[1,1]
-  attr(intercept, which = "se") <- coef_se[1, 2]
-  names(intercept) <- "(overall_intercept)"
-  return(intercept)
+  return(res)
 } # END within_intercept.plm
